// compiler.h
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "stack.h"

const int MAXIMUM = 81;            // maximum length for lines
const int SYMBOLTABLESIZE = 100;   // maximum size of symbol table
const int MEMORYSIZE = 100;        // maximum Simpletron memory

// Definition of structure for symbol table entries
struct TableEntry {
   int location;              // SML memory location 00 to 99
   char type;                 // 'C' = constant, 'V' = variable,
   int symbol;                // or 'L' = line number
};

// Function prototypes for compiler functions
int checkSymbolTable( TableEntry *, int, char );
int checkOperand( TableEntry *, char *, int *, int *, int * );
void addToSymbolTable( char, int, int, TableEntry *, int );
void addLineToFlags( int, int, int *, int *, const int * );
void compile( ifstream &, char * );
void printOutput( const int [], char * );
void lineNumber( char *, TableEntry *, int, int );
void initArrays( int *, int *, TableEntry * );
void firstPass( int *, int *, TableEntry *, ifstream & );
void secondPass( int *, int *, TableEntry * );
void separateToken( char *, int *, int *, TableEntry *, int *, int * );
void keyWord( char *, int *, int *, TableEntry *, int *, int * );
void keyLet( char *, int *, TableEntry *, int *, int * );
void keyInput( char *, int *, TableEntry *, int *, int * );
void keyPrint( char *, int *, TableEntry *, int *, int * );
void keyGoto( char *, int *, int *, TableEntry *, int * );
void keyIfGoto( char *, int *, int *, TableEntry *, int *, int * );
void evaluateExpression( int, int, char *, int *, TableEntry *, int *, int *, char * );
int createLetSML( int, int, int *, int *, int *, char );
void infixToPostfix( char *, char *, int, TableEntry *, int *, int *, int * );
void evaluatePostfix( char *, int *, int *, int *, int, TableEntry * );
bool isOperator( char );
bool precedence( char, char );

// Simpletron Machine Language (SML) Operation Codes
enum SMLOperationCodes { READ = 10, WRITE = 11, LOAD = 20, STORE = 21, ADD = 30,
                         SUBTRACT = 31, DIVIDE = 32, MULTIPLY = 33, BRANCH = 40,
                         BRANCHNEG = 41, BRANCHZERO = 42, HALT = 43 };
